/*
Copyright (c) 2023 Robin E. R. Davies

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "lv2c_ui/Lv2UI.hpp"
#include "lv2c/Lv2cContainerElement.hpp"
#include "lv2c/Lv2cFlexGridElement.hpp"
#include "lv2c/Lv2cSvgElement.hpp"
#include "lv2c/Lv2cButtonElement.hpp"
#include "ToobUi.hpp"

using namespace lv2c::ui;
using namespace lv2c;
using namespace toob;

// The plugin info generated by generate_lv2c_plugin_info.
#include "ToobDelayInfo.hpp"

#define PLUGIN_CLASS ToobDelayPluginUi
#define PLUGIN_UI_URI "http://two-play.com/plugins/toob-delay-ui"
#define PLUGIN_INFO_CLASS ToobDelayPluginInfo
#define LOGO_SVG "ToobDelayLogo.svg"


 
// class declaration.
class PLUGIN_CLASS: public ToobUi {
public:
    using super=ToobUi;
    using self=PLUGIN_CLASS;
    PLUGIN_CLASS();
protected:
    virtual Lv2cContainerElement::ptr RenderClientArea() override;
};


PLUGIN_CLASS::PLUGIN_CLASS() 
: super(
    PLUGIN_INFO_CLASS::Create(),
    Lv2cSize(383,200), // default window size.
    Lv2cSize(437,120),
    LOGO_SVG
    )
{

}



Lv2cContainerElement::ptr PLUGIN_CLASS::RenderClientArea() {
    Lv2cContainerElement::ptr controlContainer = Lv2cContainerElement::Create();
    controlContainer->Style()
        .HorizontalAlignment(Lv2cAlignment::Stretch)
        .VerticalAlignment(Lv2cAlignment::Stretch)
        ;
    {
        auto controls = super::RenderControls();
        controls->Style()
            .FlexJustification(Lv2cFlexJustification::Center)
            .VerticalAlignment(Lv2cAlignment::Center)
            ;
        controlContainer->AddChild(controls);
    }
    return controlContainer;
}



// Make the plugin visible to LV2 hosts.

int linkToobDelayUi = 0; // link target to include the .o file in the final .so.

static Lv2UIRegistration<PLUGIN_CLASS> 
registration { PLUGIN_UI_URI};

