/*
Copyright (c) 2023 Robin E. R. Davies

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "lv2c_ui/Lv2UI.hpp"
#include "lv2c/Lv2cContainerElement.hpp"
#include "lv2c/Lv2cFlexGridElement.hpp"
#include "lv2c/Lv2cSvgElement.hpp"
#include "lv2c/Lv2cButtonElement.hpp"
#include "ToobUi.hpp"

using namespace lv2c::ui;
using namespace lv2c;
using namespace toob;

// The plugin info generated by generate_lv2c_plugin_info.
#include "ToobMLInfo.hpp"

#define PLUGIN_CLASS ToobMLUi
#define PLUGIN_UI_URI "http://two-play.com/plugins/toob-ml-ui"
#define PLUGIN_INFO_CLASS ToobMLInfo
#define LOGO_FILE "ToobMLLogo.svg"


 
// class declaration.
class PLUGIN_CLASS: public ToobUi {
public:
    using super=ToobUi;
    using self=PLUGIN_CLASS;
    PLUGIN_CLASS();

protected:
    virtual Lv2cElement::ptr RenderControl(Lv2cBindingProperty<double> &value, const Lv2PortInfo &portInfo);
    void UpdateGainEnable();
private:
    observer_handle_t gainObserverHandle;

    Lv2cElement::ptr gainControl;
};


PLUGIN_CLASS::PLUGIN_CLASS() 
: super(
    PLUGIN_INFO_CLASS::Create(),
    Lv2cSize(645,536), // default window size.
    Lv2cSize(470,320), // default help window size.
    LOGO_FILE
    
    )
{
    gainObserverHandle = this->GetControlProperty("gainEnable").addObserver(
            [this](double) {
                UpdateGainEnable();
            }
            );

}


void ToobMLUi::UpdateGainEnable()
{
    if (this->gainControl)
    {
        bool enabled = this->GetControlProperty("gainEnable").get() != 0;

        Lv2cVisibility visibility = enabled? Lv2cVisibility::Visible: Lv2cVisibility::Hidden;
        if (visibility != gainControl->Style().Visibility())
        {
            this->gainControl->Style().Visibility(visibility);
            this->gainControl->InvalidateLayout();
            this->gainControl->Invalidate();
        }
    }
}
Lv2cElement::ptr ToobMLUi::RenderControl(Lv2cBindingProperty<double> &value, const Lv2PortInfo &portInfo)
{
    // capture a pointer to the gain control
    Lv2cElement::ptr control = super::RenderControl(value, portInfo);
    if (portInfo.symbol() == "gain")
    {
        this->gainControl = control;
        UpdateGainEnable();
    }
    return control;
}





// Make the plugin visible to LV2 hosts.

Lv2UIRegistration<PLUGIN_CLASS> 
toobMLRegistration { PLUGIN_UI_URI};



